/* OpenSSH/Security/2 installation script                            */
/*                                                                   */
/* Place the following files in this directory:                      */
/* 1. OpenSSH distribution file, rename it to openssh.zip            */
/* 2. Vioroute package, rename it to vioroute.zip                    */
/* 3. Install.exe from Security/2                                    */
/*                                                                   */
/* Default uid/pwd = root                                            */
/*                                                                   */
/* 29.05.2004: created                                               */
/* 03.10.2004: added support for Security/2, made fully unattended   */
/* 26.10.2004: added object creation + small improvements            */
/* 29.10.2004: uninstallation improved                               */
/* 06.11.2004: more (un)installation improvements                    */

parse arg target source mode

/* get additional parameters from environment */
product_log       = value("PRODUCT_LOG"      , ,"OS2ENVIRONMENT")
product_port      = value("PRODUCT_PORT"     , ,"OS2ENVIRONMENT")
product_string    = value("PRODUCT_STRING"   , ,"OS2ENVIRONMENT")
inst_dir          = value("INST_DIR"         , ,"OS2ENVIRONMENT")
hostname          = value("HOSTNAME"         , ,"OS2ENVIRONMENT")

/* files/dirs */
cfgsys = target'\CONFIG.SYS'
etc = value('ETC',,'os2environment')
mptnbin = translate(value('PATH',,'os2environment'))
mptnbin = substr(mptnbin, pos(':\MPTN\BIN',mptnbin)-1, 11)
mptndll = substr(mptnbin, 1, 8)'DLL'
mptndoc = substr(mptnbin, 1, 8)'DOC'
'@mkdir 'mptndoc' >nul 2>>&1'
user = 'root'
home = target'\home\'user

/* check if we are in uninstall mode */
if mode = 'UNINSTALL' then do
	call uninstall
	exit
end

/* exit if package does not exist */
if stream(inst_dir'\openssh.zip', 'c', 'query exists') = '' then exit 9
Say 
Say 'Installing SSH/SSHD + Security/2 on your system.'

/* create config */
'@mkdir 'etc'\SSH >nul 2>>&1'
'@unzip -ojC 'inst_dir'\openssh.zip etc\ssh\* -d 'etc'\ssh\. >> 'product_log
'@unzip -ojC 'inst_dir'\vioroute.zip term.cfg -d 'etc'\ssh\. >> 'product_log
port = product_port
sshdcfg = etc'\ssh\sshd_config' 
'@copy 'sshdcfg' 'etc'\ssh\sshd_config.org' 
'@echo Port 'port' > 'sshdcfg
'@echo Protocol 2  >> 'sshdcfg
'@echo HostKey 'etc'\ssh\ssh_host_dsa_key >> 'sshdcfg
'@echo LoginGraceTime 300  >> 'sshdcfg
'@echo Banner 'etc'\ssh_banner.txt  >> 'sshdcfg
'@echo IgnoreUserKnownHosts yes  >> 'sshdcfg
'@echo PasswordAuthentication yes  >> 'sshdcfg
'@echo SyslogFacility AUTH >> 'sshdcfg
'@echo LogLevel INFO >> 'sshdcfg
'@echo PermitRootLogin yes >> 'sshdcfg
'@echo StrictModes yes >> 'sshdcfg
'@echo RSAAuthentication no >> 'sshdcfg
'@echo PubkeyAuthentication yes >> 'sshdcfg
'@echo AuthorizedKeysFile .ssh/authorized_keys >> 'sshdcfg
'@echo IgnoreRhosts yes >> 'sshdcfg
'@echo RhostsRSAAuthentication no >> 'sshdcfg
'@echo HostbasedAuthentication no >> 'sshdcfg
'@echo PermitEmptyPasswords no >> 'sshdcfg
'@echo ChallengeResponseAuthentication no >> 'sshdcfg
'@echo MaxStartups 10 >> 'sshdcfg
'@echo ClientAliveInterval 600 >> 'sshdcfg
'@echo ClientAliveCountMax 3 >> 'sshdcfg
'@echo Subsystem sftp 'mptnbin'\sftp-server.exe >> 'sshdcfg
'@echo Welcome to this secure OS/2 system > 'etc'\ssh_banner.txt'

/* unzip/copy binaries */
'@unzip -ojC 'inst_dir'\vioroute.zip *.exe       -d 'mptnbin'\.         >> 'product_log
'@unzip -ojC 'inst_dir'\vioroute.zip *.dll       -d 'mptndll'\.         >> 'product_log
'@unzip -ojC 'inst_dir'\openssh.zip  *.exe       -d 'mptnbin'\.         >> 'product_log
'@unzip -ojC 'inst_dir'\openssh.zip  *.dll       -d 'mptndll'\.         >> 'product_log
'@unzip -ojC 'inst_dir'\openssh.zip  *.inf       -d 'mptndoc'\.         >> 'product_log
'@unzip -ojC 'inst_dir'\openssh.zip  readme.eng  -d 'mptndoc'\.         >> 'product_log
'@unzip -ojC 'inst_dir'\openssh.zip  docs\*      -d 'mptndoc'\.         >> 'product_log
'@unzip -ojC 'inst_dir'\openssh.zip  OpenSSH.ICO -d 'target'\os2\.      >> 'product_log

/* create homedir */
'@mkdir 'target'\home >nul 2>>&1'
'@mkdir 'home' >nul 2>>&1'
'@mkdir 'home'\.ssh >nul 2>>&1'

/* copy/create/install sec/2 files */
aclcfg = target'\security\etc\acls' 
pwdcfg = target'\security\etc\passwd'
'@'inst_dir'\install.exe -i >> 'product_log

/* add vioroute to pwd file */
i=1
do while lines(pwdcfg)
	l.i = linein(pwdcfg)
	do j=1 to 2
		p = pos('\OS2\CMD.EXE', l.i)-1
		if p > 0 then do
			l.i = substr(l.i, 1, p-1)||space(translate(translate(mptnbin),'',':'),0)'\VIOSTART.EXE'||substr(l.i, p+13)
		end
	end
	i=i+1
end
l.0=i-1
call lineout pwdcfg
if l.0 > 0 then do
	'@del 'pwdcfg' >> 'product_log' 2>>&1'
	do i=1 to l.0
		call lineout pwdcfg, l.i
	end
end
call lineout pwdcfg


/* create keys */
phrase = product_string
if hostname = '' then do
	call value "HOSTNAME" ,"LOCALHOST" ,"OS2ENVIRONMENT"
	say 'Hostname missing, using LOCALHOST for key generation...'
	'@echo SET HOSTNAME=LOCALHOST >> 'cfgsys
end

usr_cmd = mptnbin'\rgusrk.cmd'
'@echo @echo+ > 'usr_cmd
'@echo @if "%%1" ^=^= "Q" goto generate >> 'usr_cmd
'@echo @echo This script will (re)generate your user key. >> 'usr_cmd 
'@echo @pause >> 'usr_cmd
'@echo :generate  >> 'usr_cmd
'@echo @echo Generating user key... >> 'usr_cmd
'@echo @del 'home'\.ssh\id_dsa ^>nul 2^>^>^&1 >> 'usr_cmd
'@echo @'mptnbin'\ssh-keygen.exe -q -b 2048 -t dsa -f 'home'\.ssh\id_dsa -N "'phrase'" >> 'usr_cmd
'@echo @copy 'home'\.ssh\id_dsa.pub 'home'\.ssh\authorized_keys >> 'usr_cmd
'@call 'usr_cmd' Q'

sys_cmd = mptnbin'\rgsysk.cmd'
'@echo @echo+ > 'sys_cmd
'@echo @if "%%1" ^=^= "Q" goto generate >> 'sys_cmd
'@echo @echo This script will (re)generate your system key. >> 'sys_cmd
'@echo @pause >> 'sys_cmd
'@echo :generate  >> 'sys_cmd
'@echo @echo Generating server key... >> 'sys_cmd
'@echo @del 'etc'\ssh\ssh_host_dsa_key ^>nul 2^>^>^&1 >> 'sys_cmd
'@echo @'mptnbin'\ssh-keygen.exe -q -b 2048 -t dsa -f 'etc'\ssh\ssh_host_dsa_key -N "" >> 'sys_cmd
'@call 'sys_cmd' Q'

/* inform */
'@del 'home'\updcd.txt >nul 2>>&1'
call lineout home'\updcd.txt', 'Root user is root, pwd is root, see "user.exe -?" to change it.'
call lineout home'\updcd.txt', 'Start ssh  daemon with the command "start sshd"'
call lineout home'\updcd.txt', 'Start ssh  client with the command "ssh -l 'user' <hostname>"'
call lineout home'\updcd.txt', 'Start sftp client with the command "sftp 'user'@<hostname>"'

/* create folders */
call RxFuncAdd 'SysCreateObject', 'RexxUtil', SysCreateObject
call SysCreateObject 'WPFolder', 'OpenSSH^Security/2', '<WP_DESKTOP>','OBJECTID=<OPENSSH>;'||'ALWAYSSORT=YES;ICONFILE='target'\OS2\OpenSSH.ICO','R'
call SysCreateObject 'WPFolder', 'OpenSSH^Security/2^Configuration', '<OPENSSH>','OBJECTID=<OPENSSH-C>;'||'ALWAYSSORT=YES;','R'
call SysCreateObject 'WPFolder', 'OpenSSH^Security/2^Documentation', '<OPENSSH>','OBJECTID=<OPENSSH-D>;'||'ALWAYSSORT=YES;','R'
call SysCreateObject 'WPFolder', 'OpenSSH^Security/2^Executables', '<OPENSSH>','OBJECTID=<OPENSSH-E>;'||'ALWAYSSORT=YES;','R'

/* create config objects */
call SysCreateObject 'WPProgram','OpenSSH^Configuration','<OPENSSH-C>','EXENAME=E.EXE;PARAMETERS='sshdcfg,'R'
call SysCreateObject 'WPProgram','OpenSSH^Banner','<OPENSSH-C>','EXENAME=E.EXE;PARAMETERS='etc'\ssh_banner.txt','R'
call SysCreateObject 'WPProgram','Security/2^ACL file','<OPENSSH-C>','EXENAME=E.EXE;PARAMETERS='aclcfg,'R'
call SysCreateObject 'WPProgram','Security/2^PWD file','<OPENSSH-C>','EXENAME=E.EXE;PARAMETERS='pwdcfg,'R'
call SysCreateObject 'WPProgram','Security/2^Config.Sys','<OPENSSH-C>','EXENAME=E.EXE;PARAMETERS='target'\config.sys','R'

/* create documentation objects */
docfile1 = target'\security\doc\readme'
docfile2 = target'\security\doc\acls.example.en'
if stream(docfile1, 'c', 'query exists') <> '' then call SysCreateObject 'WPProgram','Security/2^Readme','<OPENSSH-D>','EXENAME=E.EXE;PARAMETERS='docfile1,'R'
if stream(docfile2, 'c', 'query exists') <> '' then call SysCreateObject 'WPProgram','Security/2^ACL examples','<OPENSSH-D>','EXENAME=E.EXE;PARAMETERS='docfile2,'R'
call SysCreateObject 'WPProgram','OpenSSH^UpdCD Readme','<OPENSSH-D>','EXENAME=E.EXE;PARAMETERS='home'\updcd.txt','R'
call SysCreateObject 'WPProgram','OpenSSH^4 OS/2','<OPENSSH-D>','EXENAME=VIEW.EXE;PARAMETERS='mptndoc'\OPEN-SSH_4OS2.INF','R'
call SysCreateObject 'WPProgram','OpenSSH^Readme','<OPENSSH-D>','EXENAME=E.EXE;PARAMETERS='mptndoc'\readme.eng','R'
call SysCreateShadow mptndoc,'<OPENSSH-D>'

/* create executable objects */
call SysCreateObject 'WPProgram','Security/2^Log file','<OPENSSH-E>','EXENAME=E.EXE;PARAMETERS='sseslog,'R'
call SysCreateObject 'WPProgram','SSHD','<OPENSSH-E>','EXENAME=CMD.EXE;PARAMETERS=/K SSHD.EXE','R'
call SysCreateObject 'WPProgram','Regenerate^User Key','<OPENSSH-E>','EXENAME=CMD.EXE;PARAMETERS=/K rgusrk.cmd','R'
call SysCreateObject 'WPProgram','Regenerate^System Key','<OPENSSH-E>','EXENAME=CMD.EXE;PARAMETERS=/K rgsysk.cmd','R'

exit

uninstall:

	/* update config */
	q = 1
	do while lines(cfgsys)
		l.q = linein(cfgsys)
		if pos('SSES.SYS',        translate(l.q)) > 0 then iterate
		if pos('SSESCTL.EXE',     translate(l.q)) > 0 then iterate
		if pos('REM *USE IT FOR', translate(l.q)) > 0 then iterate
		q=q+1
	end
	call lineout cfgsys
	l.0=q-1
	'@copy 'cfgsys target'\os2\install\config.jjc >> 'product_log 
	'del 'cfgsys' >> 'product_log 
	do q=1 to l.0
		call lineout cfgsys, l.q
	end
	call lineout cfgsys

	/* destroy objects */
	call RxFuncAdd 'SysDestroyObject', 'RexxUtil', 'SysDestroyObject'
	call SysDestroyObject "<OPENSSH>"

	/* unlock files */
	'@call 'source'\updcd\bin\unlock.exe 'target'\security\bin\ssesctl.exe >> 'product_log
	'@call 'source'\updcd\bin\unlock.exe 'target'\security\dll\passwd.dll  >> 'product_log
	'@call 'source'\updcd\bin\unlock.exe 'target'\security\dll\sesmgr.dll  >> 'product_log

	/* delete files */
	call deldir etc'\ssh'
	call deldir target'\security'
	call deldir mptndoc
	'@del 'mptnbin'\ssh*.exe          >> 'product_log
	'@del 'mptnbin'\scp*.exe          >> 'product_log
	'@del 'mptnbin'\sftp*.exe         >> 'product_log
	'@del 'mptnbin'\vioroute*.exe     >> 'product_log
	'@del 'mptndll'\vioroute.dll      >> 'product_log
	'@del 'mptndll'\userctl.dll       >> 'product_log

return

DelDir: procedure

	parse upper arg Directory
	DirSpec = Directory'\*'

	/* delete subdirectories */
	rc = SysFileTree(DirSpec, Subdirs, 'DO', '*****', '-*---')
	do i = 1 to Subdirs.0
   		call DelDir Subdirs.i
	end

	/* delete files */
	rc = SysFileTree(DirSpec, Files, 'FO', '*****', '-*---')
	do i = 1 to Files.0
		'@del "'Files.i'"'
	end

	/* delete directory */
	'@rmdir "'Directory'"'

return
